@@ -1,5 +1,5 @@
 package App::RoboBot::Network::IRC;
-$App::RoboBot::Network::IRC::VERSION = '4.002';
+$App::RoboBot::Network::IRC::VERSION = '4.003';
 use v5.20;
 
 use namespace::autoclean;
@@ -72,6 +72,8 @@ sub BUILD {
 sub connect {
     my ($self) = @_;
 
+    $self->log->info(sprintf('Connecting to IRC server at %s:%s.', $self->host, $self->port));
+
     $self->client->reg_cb( registered => sub {
         my ($con) = @_;
 
@@ -89,6 +91,9 @@ sub connect {
     });
 
     $self->client->connect($self->host, $self->port, { nick => $self->nick->name });
+
+    $self->log->info('Connected.');
+
     $_->join for @{$self->channels};
 }
 
@@ -190,6 +195,8 @@ sub handle_message {
 sub join_channel {
     my ($self, $channel) = @_;
 
+    $self->log->info(sprintf('Joining channel #%s.', $channel->name));
+
     $self->client->send_srv( JOIN => '#' . $channel->name );
 }
 
