@@ -1,5 +1,5 @@
 package App::RoboBot::NetworkFactory;
-$App::RoboBot::NetworkFactory::VERSION = '4.002';
+$App::RoboBot::NetworkFactory::VERSION = '4.003';
 use v5.20;
 
 use namespace::autoclean;
@@ -31,16 +31,31 @@ has 'nick' => (
     required => 1,
 );
 
+has 'log' => (
+    is        => 'rw',
+    predicate => 'has_logger',
+);
+
+sub BUILD {
+    my ($self) = @_;
+
+    $self->log($self->bot->logger('core.network.factory')) unless $self->has_logger;
+}
+
 sub create {
     my ($self, $name, $net_cfg) = @_;
 
-    die 'Network name not provided.' unless defined $name && $name =~ m{^[a-z0-9_-]+$}oi;
-    die 'Configuration invalid.' unless defined $net_cfg && ref($net_cfg) eq 'HASH';
-    die 'Missing network type.' unless exists $net_cfg->{'type'};
+    $self->log->debug('Preparing to create new network object.');
+
+    die $self->log->fatal('Network name not provided.') unless defined $name && $name =~ m{^[a-z0-9_-]+$}oi;
+    die $self->log->fatal('Configuration invalid.') unless defined $net_cfg && ref($net_cfg) eq 'HASH';
+    die $self->log->fatal('Missing network type.') unless exists $net_cfg->{'type'};
 
     # Check for network-specific nick (and create object for it if present) or
     # fall back to the NetworkFactory default nick.
     if (exists $net_cfg->{'nick'}) {
+        $self->log->debug(sprintf('Network %s has a custom nick (%s). Overriding global nick for this network.', $name, $net_cfg->{'nick'}));
+
         $net_cfg->{'nick'} = App::RoboBot::Nick->new(
             config => $self->config,
             name   => $net_cfg->{'nick'},
@@ -52,13 +67,14 @@ sub create {
     return $self->create_irc($name, $net_cfg) if $net_cfg->{'type'} eq 'irc';
     return $self->create_mattermost($name, $net_cfg) if $net_cfg->{'type'} eq 'mattermost';
     return $self->create_slack($name, $net_cfg) if $net_cfg->{'type'} eq 'slack';
-    die 'Invalid network type.';
+
+    die $self->log->fatal(sprintf('Invalid network type (%s).', ($net_cfg->{'type'} // '-')));
 }
 
 sub create_irc {
     my ($self, $name, $net_cfg) = @_;
 
-#    eval 'use App::RoboBot::Network::IRC;' unless is_loaded('RoboBot::Network::IRC');
+    $self->log->debug(sprintf('IRC network creation for %s.', $name));
 
     return App::RoboBot::Network::IRC->new(
         %{$net_cfg},
@@ -71,7 +87,7 @@ sub create_irc {
 sub create_mattermost {
     my ($self, $name, $net_cfg) = @_;
 
-#    eval 'use App::RoboBot::Network::Mattermost;' unless is_loaded('RoboBot::Network::Mattermost');
+    $self->log->debug(sprintf('Mattermost network creation for %s.', $name));
 
     return App::RoboBot::Network::Mattermost->new(
         %{$net_cfg},
@@ -84,7 +100,7 @@ sub create_mattermost {
 sub create_slack {
     my ($self, $name, $net_cfg) = @_;
 
-#    eval 'use App::RoboBot::Network::Slack;' unless is_loaded('RoboBot::Network::Slack');
+    $self->log->debug(sprintf('Slack network creation for %s.', $name));
 
     return App::RoboBot::Network::Slack->new(
         %{$net_cfg},
