@@ -1,5 +1,5 @@
 package App::RoboBot::Plugin;
-$App::RoboBot::Plugin::VERSION = '4.002';
+$App::RoboBot::Plugin::VERSION = '4.003';
 use v5.20;
 
 use namespace::autoclean;
@@ -8,6 +8,7 @@ use Moose;
 use MooseX::SetOnce;
 
 use Data::Dumper;
+use Log::Log4perl;
 use Scalar::Util qw( blessed );
 
 has 'name' => (
@@ -45,6 +46,17 @@ has 'bot' => (
     traits => [qw( SetOnce )],
 );
 
+has 'log' => (
+    is        => 'rw',
+    predicate => 'has_logger',
+);
+
+sub BUILD {
+    my ($self) = @_;
+
+    $self->log(Log::Log4perl::get_logger('plugin.'.$self->ns)) unless $self->has_logger;
+}
+
 sub init {
     my ($self, $bot) = @_;
 }
@@ -65,6 +77,8 @@ sub ns {
 sub process {
     my ($self, $message, $command, $rpl, @args) = @_;
 
+    $self->log->debug(sprintf('Processing command %s request (%d arguments).', $command, scalar(@args)));
+
     # Short-circuit if this plugin is disabled on the current message's network.
     return if exists $message->network->disabled_plugins->{lc($self->name)};
 
@@ -80,8 +94,12 @@ sub process {
 
     my $method = $self->commands->{$command}{'method'};
 
+    $self->log->debug(sprintf('Command %s handled by plugin method %s.', $command, $method));
+
     # Ensure that the nick is permitted to call the function.
     if (exists $message->sender->denied_functions->{$command}) {
+        $self->log->warn(sprintf('Nick %s has been denied evaluation of the function %s on network %s.',
+            $message->sender->name, $command, $message->network->name));
         return $message->response->raise('You are not permitted to call the function (%s).', $command);
     }
 
@@ -96,6 +114,7 @@ sub process {
     # By default, we pre-process all arguments, but some plugins can opt out
     # of this to handle things like conditional evaluations or loops
     unless (exists $self->commands->{$command}{'preprocess_args'} && $self->commands->{$command}{'preprocess_args'} == 0) {
+        $self->log->debug(sprintf('Preprocessing %d arguments for command %s.', scalar(@args), $command));
         # TODO: There are much better ways of deciding how to pass a symbol
         #       that happens to have the name of a function as a function, or
         #       as a string, than this.
@@ -119,8 +138,10 @@ sub process {
     }
 
     if ($self->commands->{$command}{'keyed_args'}) {
+        $self->log->debug(sprintf('Invoking plugin method %s with keyed arguments.', $method));
         return $self->$method($message, $command, $rpl, $keyed, @args);
     } else {
+        $self->log->debug(sprintf('Invoking plugin method %s.', $method));
         return $self->$method($message, $command, $rpl, @args);
     }
 }
@@ -131,6 +152,8 @@ sub hook_before {
     return $message unless $self->has_before_hook;
 
     my $hook = $self->before_hook;
+
+    $self->log->debug(sprintf('Invoking before_hook method %s on message.', $hook));
     return $self->$hook($message);
 }
 
@@ -140,6 +163,8 @@ sub hook_after {
     return $message unless $self->has_after_hook;
 
     my $hook = $self->after_hook;
+
+    $self->log->debug(sprintf('Invoking after_hook method %s on message.', $hook));
     return $self->$hook($message);
 }
 
