@@ -1,5 +1,5 @@
 package App::RoboBot::Plugin::API::Translate;
-$App::RoboBot::Plugin::API::Translate::VERSION = '4.002';
+$App::RoboBot::Plugin::API::Translate::VERSION = '4.003';
 use v5.20;
 
 use namespace::autoclean;
@@ -237,13 +237,11 @@ sub translate_party {
         last unless defined $translation;
         last if exists $seen{$to}{$translation};
         $seen{$to}{$translation} = 1;
-printf STDERR "** #%d %s -> %s\n", $attempt, $to, $translation;
 
         $translation = $self->_do_translate($to, $from, $translation);
         last unless defined $translation;
         last if exists $seen{$from}{$translation};
         $seen{$from}{$translation} = 1;
-printf STDERR "** #%d %s -> %s\n", $attempt, $from, $translation;
 
         $equilibrium = $translation;
     }
