@@ -1,10 +1,11 @@
 package App::RoboBot::Response;
-$App::RoboBot::Response::VERSION = '4.002';
+$App::RoboBot::Response::VERSION = '4.003';
 use v5.20;
 
 use namespace::autoclean;
 
 use Moose;
+use MooseX::ClassAttribute;
 use MooseX::SetOnce;
 
 has 'content' => (
@@ -53,6 +54,17 @@ has 'bot' => (
     isa => 'App::RoboBot',
 );
 
+class_has 'log' => (
+    is        => 'rw',
+    predicate => 'has_logger',
+);
+
+sub BUILD {
+    my ($self) = @_;
+
+    $self->log($self->bot->logger('core.response')) unless $self->has_logger;
+}
+
 sub raise {
     my ($self, $format, @args) = @_;
 
@@ -63,6 +75,8 @@ sub raise {
         $self->error($format);
     }
 
+    $self->log->error(sprintf('Raising error: %s', $self->error));
+
     $self->push("Error: " . $self->error);
     $self->send;
 }
@@ -70,6 +84,8 @@ sub raise {
 sub send {
     my ($self) = @_;
 
+    $self->log->debug('Preparing to send response.');
+
     return unless $self->has_content;
 
     # Delegate sending to network protocol plugin, since length limits and
@@ -91,10 +107,14 @@ sub push {
         # one or more of their own arguments in the push(), still blessed as a
         # Data::SExpression object.
         if ($self->has_content) {
+            $self->log->debug('Appending response push data to existing content.');
             push(@{$self->content}, map { "$_" } @args);
         } else {
+            $self->log->debug('Initializing response content list with new response push data.');
             $self->content([map { "$_" } @args]);
         }
+    } else {
+        $self->log->warn('Receiving response push request with no data.');
     }
 }
 
